@@ -2,12 +2,12 @@ import json
 import os
 import sys
 
-import lazy_import
 from smac.utils import dependencies
 
-__version__ = '0.12.1'
-__author__ = 'Marius Lindauer, Matthias Feurer, Katharina Eggensperger, Joshua Marben, André Biedenkapp, Aaron Klein,'\
-    'Stefan Falkner and Frank Hutter'
+__version__ = '1.0.0'
+__author__ = 'Marius Lindauer, Katharina Eggensperger, Matthias Feurer, André Biedenkapp, ' \
+             'Difan Deng, Carolin Benjamins, René Sass ' \
+             'and Frank Hutter'
 
 
 with open(os.path.join(os.path.dirname(__file__), 'requirements.txt')) as fh:
@@ -22,14 +22,9 @@ for name, requirements in extras_require.items():
         continue
     if dependencies.are_valid_packages(requirements):
         extras_installed.add(name)
-    for requirement in requirements:
-        package_name = dependencies.RE_PATTERN.match(requirement).group('name')  # type: ignore[union-attr] # noqa F821
-        if package_name == 'scikit-optimize':
-            package_name = 'skopt'
-        lazy_import.lazy_module(package_name)
 
-if sys.version_info < (3, 5, 2):
-    raise ValueError("SMAC requires Python 3.5.2 or newer.")
+if sys.version_info < (3, 7, 0):
+    raise ValueError("SMAC requires Python 3.7.0 or newer.")
 
 
 if os.name != 'posix':
