@@ -3,7 +3,13 @@ import logging
 import typing
 import warnings
 
-import emcee
+try:
+    import emcee
+except ImportError as e:
+    raise ImportError(
+        'Could not import emcee - emcee is an optional dependency.\n'
+        'Please install it manually with `pip install emcee`.') from e
+
 import numpy as np
 
 from smac.configspace import ConfigurationSpace
@@ -11,8 +17,8 @@ from smac.epm.base_gp import BaseModel
 from smac.epm.gaussian_process import GaussianProcess
 from smac.epm.gp_base_prior import Prior
 
-from skopt.learning.gaussian_process.kernels import Kernel
-from skopt.learning.gaussian_process import GaussianProcessRegressor
+from sklearn.gaussian_process.kernels import Kernel
+from sklearn.gaussian_process import GaussianProcessRegressor
 
 logger = logging.getLogger(__name__)
 
@@ -184,7 +190,7 @@ class GaussianProcessMCMC(BaseModel):
                     self.p0, _, _ = sampler.run_mcmc(self.p0, self.chain_length)
 
                 # Take the last samples from each walker
-                self.hypers = sampler.get_chain()[:, -1]
+                self.hypers = sampler.get_chain()[-1]
             elif self.mcmc_sampler == 'nuts':
                 # Originally published as:
                 # http://www.stat.columbia.edu/~gelman/research/published/nuts.pdf
@@ -285,7 +291,6 @@ class GaussianProcessMCMC(BaseModel):
             optimizer=None,
             n_restarts_optimizer=-1,  # Do not use scikit-learn's optimization routine
             alpha=0,  # Governed by the kernel
-            noise=None,
         )
 
     def _ll(self, theta: np.ndarray) -> float:
