@@ -1,15 +1,14 @@
-from inspect import signature
+from inspect import signature, Signature
 import math
-from typing import Optional
+from typing import Optional, Union, Tuple, List, Callable, Dict, Any
 
 import numpy as np
-import sklearn.gaussian_process.kernels
+import sklearn.gaussian_process.kernels as kernels
 import scipy.optimize
 import scipy.spatial.distance
 import scipy.special
 
-from lazy_import import lazy_module
-kernels = lazy_module('skopt.learning.gaussian_process.kernels')
+from smac.epm.gp_base_prior import Prior
 
 # This file contains almost no type annotations to simplify comparing it to the original scikit-learn version!
 
@@ -27,11 +26,23 @@ def get_conditional_hyperparameters(X: np.ndarray, Y: Optional[np.ndarray]) -> n
 
 class MagicMixin:
 
-    prior = None
+    # This is a mixin for a kernel to override functions of the kernel. Because it overrides functions of the kernel,
+    # it needs to be placed first in the inheritance hierarchy. For this reason it is not possible to subclass the
+    # Mixin from the kernel class because this will prevent it from being instantiatable. Therefore, mypy won't know
+    # about anything related to the superclass and I had to add a few type:ignore statements when accessing a member
+    # that is declared in the superclass such as self.has_conditions, self._call, super().get_params etc.
 
-    def __call__(self, X, Y=None, eval_gradient=False, active=None):
+    prior = None  # type: Optional[Prior]
 
-        if active is None and self.has_conditions:
+    def __call__(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: Optional[np.ndarray] = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
+
+        if active is None and self.has_conditions:  # type: ignore[attr-defined] # noqa F821
             if self.operate_on is None:
                 active = get_conditional_hyperparameters(X, Y)
             else:
@@ -41,10 +52,10 @@ class MagicMixin:
                     active = get_conditional_hyperparameters(X[:, self.operate_on], Y[:, self.operate_on])
 
         if self.operate_on is None:
-            rval = self._call(X, Y, eval_gradient, active)
+            rval = self._call(X, Y, eval_gradient, active)  # type: ignore[attr-defined] # noqa F821
         else:
             if Y is None:
-                rval = self._call(
+                rval = self._call(  # type: ignore[attr-defined] # noqa F821
                     X=X[:, self.operate_on].reshape([-1, self.len_active]),
                     Y=None,
                     eval_gradient=eval_gradient,
@@ -52,7 +63,7 @@ class MagicMixin:
                 )
                 X = X[:, self.operate_on].reshape((-1, self.len_active))
             else:
-                rval = self._call(
+                rval = self._call(  # type: ignore[attr-defined] # noqa F821
                     X=X[:, self.operate_on].reshape([-1, self.len_active]),
                     Y=Y[:, self.operate_on].reshape([-1, self.len_active]),
                     eval_gradient=eval_gradient,
@@ -63,38 +74,40 @@ class MagicMixin:
 
         return rval
 
-    def __add__(self, b):
+    def __add__(self, b: Union[kernels.Kernel, float]) -> kernels.Sum:
         if not isinstance(b, kernels.Kernel):
             return Sum(self, ConstantKernel(b))
         return Sum(self, b)
 
-    def __radd__(self, b):
+    def __radd__(self, b: Union[kernels.Kernel, float]) -> kernels.Sum:
         if not isinstance(b, kernels.Kernel):
             return Sum(ConstantKernel(b), self)
         return Sum(b, self)
 
-    def __mul__(self, b):
+    def __mul__(self, b: Union[kernels.Kernel, float]) -> kernels.Product:
         if not isinstance(b, kernels.Kernel):
             return Product(self, ConstantKernel(b))
         return Product(self, b)
 
-    def __rmul__(self, b):
+    def __rmul__(self, b: Union[kernels.Kernel, float]) -> kernels.Product:
         if not isinstance(b, kernels.Kernel):
             return Product(ConstantKernel(b), self)
         return Product(b, self)
 
-    def _signature(self, func):
+    def _signature(self, func: Callable) -> Signature:
         try:
-            sig = self._signature_cache.get(func)
+            sig_ = self._signature_cache.get(func)  # type: Optional[Signature]
         except AttributeError:
-            self._signature_cache = dict()
-            sig = None
-        if sig is None:
+            self._signature_cache = {}  # type: Dict[Callable, Signature]
+            sig_ = None
+        if sig_ is None:
             sig = signature(func)
             self._signature_cache[func] = sig
-        return sig
+            return sig
+        else:
+            return sig_
 
-    def get_params(self, deep=True):
+    def get_params(self, deep: bool = True) -> Dict[str, Any]:
         """Get parameters of this kernel.
 
         Parameters
@@ -113,33 +126,35 @@ class MagicMixin:
         try:
             args = self._args_cache
         except AttributeError:
-            tmp = super().get_params(deep)
+            # ignore[misc] looks like it catches all kinds of errors, but misc is actually a category from mypy:
+            # https://mypy.readthedocs.io/en/latest/error_code_list.html#miscellaneous-checks-misc
+            tmp = super().get_params(deep)  # type: ignore[misc] # noqa F821
             args = list(tmp.keys())
             # Sum and Product do not clone the 'has_conditions' attribute by default. Instead of changing their
             # get_params() method, we simply add the attribute here!
             if 'has_conditions' not in args:
                 args.append('has_conditions')
-            self._args_cache = args
+            self._args_cache = args  # type: List[Union[str, Any]]
 
         for arg in args:
             params[arg] = getattr(self, arg, None)
         return params
 
     @property
-    def hyperparameters(self):
+    def hyperparameters(self) -> List[kernels.Hyperparameter]:
         """Returns a list of all hyperparameter specifications."""
         try:
             return self._hyperparameters_cache
         except AttributeError:
             pass
 
-        r = super().hyperparameters
-        self._hyperparameters_cache = r
+        r = super().hyperparameters  # type: ignore[misc] # noqa F821
+        self._hyperparameters_cache = r  # type: List[kernels.Hyperparameter]
 
         return r
 
     @property
-    def n_dims(self):
+    def n_dims(self) -> int:
         """Returns the number of non-fixed hyperparameters of the kernel."""
 
         try:
@@ -147,10 +162,11 @@ class MagicMixin:
         except AttributeError:
             pass
 
-        self._n_dims_cache = super().n_dims
+        self._n_dims_cache = -1  # type: int # I cannot use `varname: type = value` syntax because that's >=Python3.6
+        self._n_dims_cache = super().n_dims  # type: ignore[misc] # noqa F821
         return self._n_dims_cache
 
-    def clone_with_theta(self, theta):
+    def clone_with_theta(self, theta: np.ndarray) -> kernels.Kernel:
         """Returns a clone of self with given hyperparameters theta.
 
         Parameters
@@ -161,7 +177,7 @@ class MagicMixin:
         self.theta = theta
         return self
 
-    def set_active_dims(self, operate_on=None):
+    def set_active_dims(self, operate_on: Optional[np.ndarray] = None) -> None:
         """Sets dimensions this kernel should work on
 
         Parameters
@@ -173,8 +189,8 @@ class MagicMixin:
                 raise TypeError('argument operate_on needs to be of type np.ndarray, but is %s' % type(operate_on))
             if operate_on.dtype != np.int:
                 raise ValueError('dtype of argument operate_on needs to be np.int, but is %s' % operate_on.dtype)
-            self.operate_on = operate_on
-            self.len_active = len(operate_on)
+            self.operate_on = operate_on  # type: Optional[np.ndarray]
+            self.len_active = len(operate_on)  # type: Optional[int]
         else:
             self.operate_on = None
             self.len_active = None
@@ -182,12 +198,24 @@ class MagicMixin:
 
 class Sum(MagicMixin, kernels.Sum):
 
-    def __init__(self, k1, k2, operate_on=None, has_conditions=False):
+    def __init__(
+        self,
+        k1: kernels.Kernel,
+        k2: kernels.Kernel,
+        operate_on: np.ndarray = None,
+        has_conditions: bool = False,
+    ) -> None:
         super(Sum, self).__init__(k1=k1, k2=k2)
         self.set_active_dims(operate_on)
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: np.ndarray = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """Return the kernel k(X, Y) and optionally its gradient.
 
         Parameters
@@ -226,12 +254,24 @@ class Sum(MagicMixin, kernels.Sum):
 
 class Product(MagicMixin, kernels.Product):
 
-    def __init__(self, k1, k2, operate_on=None, has_conditions=False):
+    def __init__(
+        self,
+        k1: kernels.Kernel,
+        k2: kernels.Kernel,
+        operate_on: np.ndarray = None,
+        has_conditions: bool = False,
+    ) -> None:
         super(Product, self).__init__(k1=k1, k2=k2)
         self.set_active_dims(operate_on)
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: np.ndarray = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """Return the kernel k(X, Y) and optionally its gradient.
 
         Parameters
@@ -272,19 +312,26 @@ class Product(MagicMixin, kernels.Product):
 class ConstantKernel(MagicMixin, kernels.ConstantKernel):
 
     def __init__(
-            self,
-            constant_value=1.0,
-            constant_value_bounds=(1e-5, 1e5),
-            operate_on=None,
-            prior=None,
-            has_conditions=False,
-    ):
+        self,
+        constant_value: float = 1.0,
+        constant_value_bounds: Tuple[float, float] = (1e-5, 1e5),
+        operate_on: Optional[np.ndarray] = None,
+        prior: Optional[Prior] = None,
+        has_conditions: bool = False,
+    ) -> None:
+
         super(ConstantKernel, self).__init__(constant_value=constant_value, constant_value_bounds=constant_value_bounds)
         self.set_active_dims(operate_on)
         self.prior = prior
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: Optional[np.ndarray] = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """Return the kernel k(X, Y) and optionally its gradient.
 
         Parameters
@@ -336,20 +383,28 @@ class Matern(MagicMixin, kernels.Matern):
 
     def __init__(
         self,
-        length_scale=1.0,
-        length_scale_bounds=(1e-5, 1e5),
-        nu=1.5,
-        operate_on=None,
-        prior=None,
-        has_conditions=False
-    ):
+        length_scale: Union[float, Tuple[float, ...]] = 1.0,
+        length_scale_bounds: Union[Tuple[float, float], List[Tuple[float, float]]] = (1e-5, 1e5),
+        nu: float = 1.5,
+        operate_on: Optional[np.ndarray] = None,
+        prior: Optional[Prior] = None,
+        has_conditions: bool = False,
+    ) -> None:
+
         super(Matern, self).__init__(length_scale=length_scale, length_scale_bounds=length_scale_bounds, nu=nu)
         self.set_active_dims(operate_on)
         self.prior = prior
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
-        """Return the kernel k(X, Y) and optionally its gradient.
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: Optional[np.ndarray] = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
+        """ Return the kernel k(X, Y) and optionally its gradient.
+
         Parameters
         ----------
         X : array, shape (n_samples_X, n_features)
@@ -362,6 +417,7 @@ class Matern(MagicMixin, kernels.Matern):
             hyperparameter is determined. Only supported when Y is None.
         active : np.ndarray (n_samples_X, n_features) (optional)
             Boolean array specifying which hyperparameters are active.
+
         Returns
         -------
         K : array, shape (n_samples_X, n_samples_Y)
@@ -373,7 +429,7 @@ class Matern(MagicMixin, kernels.Matern):
         """
 
         X = np.atleast_2d(X)
-        length_scale = sklearn.gaussian_process.kernels._check_length_scale(X, self.length_scale)
+        length_scale = kernels._check_length_scale(X, self.length_scale)
 
         if Y is None:
             dists = scipy.spatial.distance.pdist(X / length_scale, metric='euclidean')
@@ -444,19 +500,28 @@ class RBF(MagicMixin, kernels.RBF):
 
     def __init__(
         self,
-        length_scale=1.0,
-        length_scale_bounds=(1e-5, 1e5),
-        operate_on=None,
-        prior=None,
-        has_conditions=False,
-    ):
+        length_scale: Union[float, Tuple[float, ...]] = 1.0,
+        length_scale_bounds: Union[Tuple[float, float], List[Tuple[float, float]]] = (1e-5, 1e5),
+        operate_on: Optional[np.ndarray] = None,
+        prior: Optional[Prior] = None,
+        has_conditions: bool = False,
+    ) -> None:
+
         super(RBF, self).__init__(length_scale=length_scale, length_scale_bounds=length_scale_bounds)
         self.set_active_dims(operate_on)
         self.prior = prior
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: Optional[np.ndarray] = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
+
         """Return the kernel k(X, Y) and optionally its gradient.
+
         Parameters
         ----------
         X : array, shape (n_samples_X, n_features)
@@ -469,6 +534,7 @@ class RBF(MagicMixin, kernels.RBF):
             hyperparameter is determined. Only supported when Y is None.
         active : np.ndarray (n_samples_X, n_features) (optional)
             Boolean array specifying which hyperparameters are active.
+
         Returns
         -------
         K : array, shape (n_samples_X, n_samples_Y)
@@ -480,7 +546,7 @@ class RBF(MagicMixin, kernels.RBF):
         """
 
         X = np.atleast_2d(X)
-        length_scale = sklearn.gaussian_process.kernels._check_length_scale(X, self.length_scale)
+        length_scale = kernels._check_length_scale(X, self.length_scale)
 
         if Y is None:
             dists = scipy.spatial.distance.pdist(X / length_scale, metric='sqeuclidean')
@@ -507,31 +573,38 @@ class RBF(MagicMixin, kernels.RBF):
                 return K, K_gradient
             elif self.anisotropic:
                 # We need to recompute the pairwise dimension-wise distances
-                K_gradient = (X[:, np.newaxis, :] - X[np.newaxis, :, :]) ** 2 \
-                             / (length_scale ** 2)
+                K_gradient = (X[:, np.newaxis, :] - X[np.newaxis, :, :]) ** 2 / (length_scale ** 2)
                 K_gradient *= K[..., np.newaxis]
                 return K, K_gradient
-        else:
-            return K
+
+        return K
 
 
 class WhiteKernel(MagicMixin, kernels.WhiteKernel):
 
     def __init__(
         self,
-        noise_level=1.0,
-        noise_level_bounds=(1e-5, 1e5),
-        operate_on=None,
-        prior=None,
-        has_conditions=False,
-    ):
+        noise_level: Union[float, Tuple[float, ...]] = 1.0,
+        noise_level_bounds: Union[Tuple[float, float], List[Tuple[float, float]]] = (1e-5, 1e5),
+        operate_on: Optional[np.ndarray] = None,
+        prior: Optional[Prior] = None,
+        has_conditions: bool = False,
+    ) -> None:
+
         super(WhiteKernel, self).__init__(noise_level=noise_level, noise_level_bounds=noise_level_bounds)
         self.set_active_dims(operate_on)
         self.prior = prior
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
-        """Return the kernel k(X, Y) and optionally its gradient.
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: Optional[np.ndarray] = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
+        """ Return the kernel k(X, Y) and optionally its gradient.
+
         Parameters
         ----------
         X : array, shape (n_samples_X, n_features)
@@ -544,6 +617,7 @@ class WhiteKernel(MagicMixin, kernels.WhiteKernel):
             hyperparameter is determined. Only supported when Y is None.
         active : np.ndarray (n_samples_X, n_features) (optional)
             Boolean array specifying which hyperparameters are active.
+
         Returns
         -------
         K : array, shape (n_samples_X, n_samples_Y)
@@ -567,8 +641,7 @@ class WhiteKernel(MagicMixin, kernels.WhiteKernel):
 
             if eval_gradient:
                 if not self.hyperparameter_noise_level.fixed:
-                    return (K, self.noise_level
-                            * np.eye(X.shape[0])[:, :, np.newaxis])
+                    return (K, self.noise_level * np.eye(X.shape[0])[:, :, np.newaxis])
                 else:
                     return K, np.empty((X.shape[0], X.shape[0], 0))
             else:
@@ -577,22 +650,37 @@ class WhiteKernel(MagicMixin, kernels.WhiteKernel):
             return np.zeros((X.shape[0], Y.shape[0]))
 
 
-class HammingKernel(MagicMixin, kernels.HammingKernel):
+class HammingKernel(MagicMixin, kernels.StationaryKernelMixin, kernels.NormalizedKernelMixin, kernels.Kernel):
 
     def __init__(
         self,
-        length_scale=1.0,
-        length_scale_bounds=(1e-5, 1e5),
-        operate_on=None,
-        prior=None,
-        has_conditions=False,
-    ):
-        super(HammingKernel, self).__init__(length_scale=length_scale, length_scale_bounds=length_scale_bounds)
+        length_scale: Union[float, Tuple[float, ...]] = 1.0,
+        length_scale_bounds: Union[Tuple[float, float], List[Tuple[float, float]]] = (1e-5, 1e5),
+        operate_on: Optional[np.ndarray] = None,
+        prior: Optional[Prior] = None,
+        has_conditions: bool = False,
+    ) -> None:
+        self.length_scale = length_scale
+        self.length_scale_bounds = length_scale_bounds
         self.set_active_dims(operate_on)
         self.prior = prior
         self.has_conditions = has_conditions
 
-    def _call(self, X, Y=None, eval_gradient=False, active=None):
+    @property
+    def hyperparameter_length_scale(self) -> kernels.Hyperparameter:
+        length_scale = self.length_scale
+        anisotropic = np.iterable(length_scale) and len(length_scale) > 1  # type: ignore
+        if anisotropic:
+            return kernels.Hyperparameter("length_scale", "numeric", self.length_scale_bounds, len(length_scale))  # type: ignore  # noqa: E501
+        return kernels.Hyperparameter("length_scale", "numeric", self.length_scale_bounds)
+
+    def _call(
+        self,
+        X: np.ndarray,
+        Y: Optional[np.ndarray] = None,
+        eval_gradient: bool = False,
+        active: Optional[np.ndarray] = None,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """Return the kernel k(X, Y) and optionally its gradient.
 
         Parameters
@@ -625,7 +713,7 @@ class HammingKernel(MagicMixin, kernels.HammingKernel):
         """
 
         X = np.atleast_2d(X)
-        length_scale = sklearn.gaussian_process.kernels._check_length_scale(X, self.length_scale)
+        length_scale = kernels._check_length_scale(X, self.length_scale)
 
         if Y is None:
             Y = X
@@ -635,7 +723,7 @@ class HammingKernel(MagicMixin, kernels.HammingKernel):
             Y = np.atleast_2d(Y)
 
         indicator = np.expand_dims(X, axis=1) != Y
-        K = (-1/(2*length_scale**2) * indicator).sum(axis=2)
+        K = (-1 / (2 * length_scale**2) * indicator).sum(axis=2)
         K = np.exp(K)
 
         if active is not None:
