@@ -422,13 +422,10 @@ class PI(AbstractAcquisitionFunction):
     def __init__(self,
                  model: AbstractEPM,
                  par: float = 0.0):
+        r"""Computes the probability of improvement for a given x over the best so far value as acquisition value.
 
-        r"""Computes the probability of improvement for a given x over the best so far value as
-        acquisition value.
-
-        :math:`P(f_{t+1}(\mathbf{X})\geq f(\mathbf{X^+})) :=
-        \Phi(\frac{\mu(\mathbf{X}) - f(\mathbf{X^+})}{\sigma(\mathbf{X})})`,
-        with :math:`f(X^+)` as the best location and :math:`\Phi` the cdf of the standard normal
+        :math:`P(f_{t+1}(\mathbf{X})\geq f(\mathbf{X^+}))` :math:`:= \Phi(\\frac{ \mu(\mathbf{X})-f(\mathbf{X^+}) }
+        { \sigma(\mathbf{X}) })` with :math:`f(X^+)` as the incumbent and :math:`\Phi` the cdf of the standard normal
 
         Parameters
         ----------
@@ -487,7 +484,7 @@ class LCB(AbstractAcquisitionFunction):
         model : AbstractEPM
             A model that implements at least
                  - predict_marginalized_over_instances(X)
-        par : float, default=0.0
+        par : float, default=1.0
             Controls the balance between exploration and exploitation of the
             acquisition function.
         """
@@ -520,3 +517,51 @@ class LCB(AbstractAcquisitionFunction):
         std = np.sqrt(var_)
         beta = 2 * np.log((X.shape[1] * self.num_data**2) / self.par)
         return -(m - np.sqrt(beta) * std)
+
+
+class TS(AbstractAcquisitionFunction):
+    def __init__(self,
+                 model: AbstractEPM,
+                 par: float = 0.0):
+
+        r"""Do a Thompson Sampling for a given x over the best so far value as
+        acquisition value.
+        :math:`TS(X) ~ \mathcal{N}(\mu(\mathbf{X}),\sigma(\mathbf{X}))'
+        Returns -TS(X) as the acquisition_function optimizer maximizes the acquisition value.
+        Parameters
+        ----------
+        model : AbstractEPM
+            A model that implements at least
+                 - predict_marginalized_over_instances(X)
+        par : float, default=0.0
+            TS does not require par here, we only wants to make it consistent with other acquisition functions
+        """
+        super(TS, self).__init__(model)
+        self.long_name = 'Thompson Sampling'
+        self.par = par
+        self.num_data = None
+        self._required_updates = ('model', )
+
+    def _compute(self, X: np.ndarray) -> np.ndarray:
+        """Sample a new value from a gaussian distribution whose mean and covariance values are given by model
+        Parameters
+        ----------
+        X: np.ndarray(N, D)
+           Points to be evaluated where we could sample a value. N is the number of points and D the dimension
+           for the points
+        Returns
+        -------
+        np.ndarray(N,1)
+            negative sample value of X
+        """
+        if len(X.shape) == 1:
+            X = X[:, np.newaxis]
+        sample_function = getattr(self.model, "sample_functions", None)
+        if callable(sample_function):
+            return - sample_function(X, n_funcs=1)
+
+        m, var_ = self.model.predict_marginalized_over_instances(X)
+        rng = getattr(self.model, 'rng', np.random.RandomState(self.model.seed))
+        m = m.flatten()
+        var_ = np.diag(var_.flatten())
+        return - rng.multivariate_normal(m, var_, 1).T
