@@ -6,7 +6,7 @@ import typing
 import numpy as np
 
 from smac.configspace import Configuration, ConfigurationSpace
-from smac.tae.execute_ta_run import StatusType
+from smac.tae import StatusType
 from smac.utils.logging import PickableLoggerAdapter
 
 __author__ = "Marius Lindauer"
@@ -32,6 +32,34 @@ class RunKey(collections.namedtuple('RunKey', ['config_id', 'instance_id', 'seed
         return super().__new__(cls, config_id, instance_id, seed, budget)
 
 
+# NOTE class instead of collection to have a default value for budget/source_id in RunInfo
+class RunInfo(
+    collections.namedtuple(
+        'RunInfo',
+        ['config', 'instance', 'instance_specific', 'seed', 'cutoff', 'capped', 'budget', 'source_id']
+    )
+):
+    __slots__ = ()
+
+    def __new__(
+        cls,  # No type annotation because the 1st argument for a namedtuble is always the class type,
+              # see https://docs.python.org/3/reference/datamodel.html#object.__new__
+        config: Configuration,
+        instance: typing.Optional[str],
+        instance_specific: str,
+        seed: int,
+        cutoff: typing.Optional[float],
+        capped: bool,
+        budget: float = 0.0,
+        # In the context of parallel runs, one will have multiple suppliers of
+        # configurations. source_id is a new mechanism to track what entity launched
+        # this configuration
+        source_id: int = 0,
+    ) -> 'RunInfo':
+        return super().__new__(cls, config, instance, instance_specific, seed,
+                               cutoff, capped, budget, source_id)
+
+
 InstSeedKey = collections.namedtuple(
     'InstSeedKey', ['instance', 'seed'])
 
@@ -41,12 +69,12 @@ InstSeedBudgetKey = collections.namedtuple(
 
 
 RunValue = collections.namedtuple(
-    'RunValue', ['cost', 'time', 'status', 'additional_info'])
+    'RunValue', ['cost', 'time', 'status', 'starttime', 'endtime', 'additional_info'])
 
 
 class EnumEncoder(json.JSONEncoder):
     """Custom encoder for enum-serialization
-    (implemented for StatusType from tae/execute_ta_run).
+    (implemented for StatusType from tae).
     Using encoder implied using object_hook as defined in StatusType
     to deserialize from json.
     """
@@ -168,9 +196,12 @@ class RunHistory(object):
         status: StatusType,
         instance_id: typing.Optional[str] = None,
         seed: typing.Optional[int] = None,
-        budget: float = 0,
+        budget: float = 0.0,
+        starttime: float = 0.0,
+        endtime: float = 0.0,
         additional_info: typing.Optional[typing.Dict] = None,
         origin: DataOrigin = DataOrigin.INTERNAL,
+        force_update: bool = False,
     ) -> None:
         """Adds a data of a new target algorithm (TA) run;
         it will update data if the same key values are used
@@ -192,11 +223,17 @@ class RunHistory(object):
                 Random seed used by TA (default: None)
             budget: float
                 budget (cutoff) used in intensifier to limit TA (default: 0)
+            starttime: float
+                starting timestamp of TA evaluation
+            endtime: float
+                ending timestamp of TA evaluation
             additional_info: dict
                 Additional run infos (could include further returned
                 information from TA or fields such as start time and host_id)
             origin: DataOrigin
                 Defines how data will be used.
+            force_update: bool (default: False)
+                Forces the addition of a config to the history
         """
 
         if config is None:
@@ -216,13 +253,28 @@ class RunHistory(object):
         else:
             config_id = typing.cast(int, config_id_tmp)
 
-        # Construct keys and values for the data dictionary
         k = RunKey(config_id, instance_id, seed, budget)
-        v = RunValue(cost, time, status, additional_info)
+        v = RunValue(cost, time, status, starttime, endtime, additional_info)
+        # Construct keys and values for the data dictionary
+        for key, value in (
+            ('config', config.get_dictionary()),
+            ('config_id', config_id),
+            ('instance_id', instance_id),
+            ('seed', seed),
+            ('budget', budget),
+            ('cost', cost),
+            ('time', time),
+            ('status', status),
+            ('starttime', starttime),
+            ('endtime', endtime),
+            ('additional_info', additional_info),
+            ('origin', config.origin),
+        ):
+            self._check_json_serializable(key, value, EnumEncoder, k, v)
 
         # Each runkey is supposed to be used only once. Repeated tries to add
         # the same runkey will be ignored silently if not capped.
-        if self.overwrite_existing_runs or self.data.get(k) is None:
+        if self.overwrite_existing_runs or force_update or self.data.get(k) is None:
             self._add(k, v, status, origin)
         elif status != StatusType.CAPPED and self.data[k].status == StatusType.CAPPED:
             # overwrite capped runs with uncapped runs
@@ -231,6 +283,22 @@ class RunHistory(object):
             # overwrite if censored with a larger cutoff
             self._add(k, v, status, origin)
 
+    def _check_json_serializable(
+        self,
+        key: str,
+        obj: typing.Any,
+        encoder: typing.Type[json.JSONEncoder],
+        runkey: RunKey,
+        runvalue: RunValue
+    ) -> None:
+        try:
+            json.dumps(obj, cls=encoder)
+        except Exception as e:
+            raise ValueError(
+                "Cannot add %s: %s of type %s to runhistory because it raises an error during JSON encoding, "
+                "please see the error above.\nRunKey: %s\nRunValue %s" % (key, str(obj), type(obj), runkey, runvalue)
+            ) from e
+
     def _add(self, k: RunKey, v: RunValue, status: StatusType,
              origin: DataOrigin) -> None:
         """Actual function to add new entry to data structures
@@ -241,8 +309,10 @@ class RunHistory(object):
         self.data[k] = v
         self.external[k] = origin
 
+        # Capped data is added above
+        # Do not register the cost until the run has completed
         if origin in (DataOrigin.INTERNAL, DataOrigin.EXTERNAL_SAME_INSTANCES) \
-                and status != StatusType.CAPPED:  # Capped data is added above
+                and status not in [StatusType.CAPPED, StatusType.RUNNING]:
             # also add to fast data structure
             is_k = InstSeedKey(k.instance_id, k.seed)
             self._configid_to_inst_seed_budget[k.config_id] = self._configid_to_inst_seed_budget.get(k.config_id, {})
@@ -368,8 +438,8 @@ class RunHistory(object):
         """
         Return all configs in this RunHistory object that have been run on one of these budgets
 
-        Parameter
-        ---------
+        Parameters
+        ----------
             budget_subset: list
 
         Returns
@@ -487,7 +557,9 @@ class RunHistory(object):
                      instance_id=k[1],
                      seed=int(k[2]),
                      budget=float(k[3]) if len(k) == 4 else 0,
-                     additional_info=v[3])
+                     starttime=v[3],
+                     endtime=v[4],
+                     additional_info=v[5])
 
     def update_from_json(
         self,
@@ -533,10 +605,10 @@ class RunHistory(object):
         #  the ID
         for key, value in runhistory.data.items():
             config_id, instance_id, seed, budget = key
-            cost, time, status, additional_info = value
+            cost, time, status, start, end, additional_info = value
             config = runhistory.ids_config[config_id]
             self.add(config=config, cost=cost, time=time,
-                     status=status, instance_id=instance_id,
+                     status=status, instance_id=instance_id, starttime=start, endtime=end,
                      seed=seed, budget=budget, additional_info=additional_info,
                      origin=origin)
 
