@@ -15,8 +15,6 @@ from smac.configspace import (
 from smac.epm.gaussian_process import GaussianProcess
 from smac.epm.gp_base_prior import HorseshoePrior, LognormalPrior
 
-from test import requires_extra
-
 
 def get_gp(n_dimensions, rs, noise=1e-3, normalize_y=True) -> GaussianProcess:
     from smac.epm.gp_kernels import ConstantKernel, Matern, WhiteKernel
@@ -130,7 +128,6 @@ def get_mixed_gp(cat_dims, cont_dims, rs, noise=1e-3, normalize_y=True):
     return model
 
 
-@requires_extra('gp')
 class TestGP(unittest.TestCase):
 
     def test_predict_wrong_X_dimensions(self):
@@ -143,16 +140,16 @@ class TestGP(unittest.TestCase):
 
         for model in (get_gp(n_dims, rs), get_mixed_gp(cat_dims, cont_dims, rs)):
             X = rs.rand(10)
-            self.assertRaisesRegexp(ValueError, "Expected 2d array, got 1d array!",
-                                    model.predict, X)
+            self.assertRaisesRegex(ValueError, "Expected 2d array, got 1d array!",
+                                   model.predict, X)
             X = rs.rand(10, 10, 10)
-            self.assertRaisesRegexp(ValueError, "Expected 2d array, got 3d array!",
-                                    model.predict, X)
+            self.assertRaisesRegex(ValueError, "Expected 2d array, got 3d array!",
+                                   model.predict, X)
 
             X = rs.rand(10, 5)
-            self.assertRaisesRegexp(ValueError, "Rows in X should have 10 entries "
-                                                "but have 5!",
-                                    model.predict, X)
+            self.assertRaisesRegex(ValueError, "Rows in X should have 10 entries "
+                                               "but have 5!",
+                                   model.predict, X)
 
     def test_predict(self):
         rs = np.random.RandomState(1)
@@ -191,7 +188,7 @@ class TestGP(unittest.TestCase):
         self.assertFalse(np.any(theta_ == fixture))
         np.testing.assert_array_almost_equal(theta, theta_)
 
-    @unittest.mock.patch('skopt.learning.gaussian_process.GaussianProcessRegressor.fit')
+    @unittest.mock.patch('sklearn.gaussian_process.GaussianProcessRegressor.fit')
     def test_train_continue_on_linalg_error(self, fit_mock):
         # Check that training does not stop on a linalg error, but that uncertainty is increased!
 
@@ -215,14 +212,14 @@ class TestGP(unittest.TestCase):
         model._train(X[:10], Y[:10], do_optimize=False)
         self.assertAlmostEqual(np.exp(model.gp.kernel.theta[-1]), fixture + 10)
 
-    @unittest.mock.patch('skopt.learning.gaussian_process.GaussianProcessRegressor.log_marginal_likelihood')
+    @unittest.mock.patch('sklearn.gaussian_process.GaussianProcessRegressor.log_marginal_likelihood')
     def test_train_continue_on_linalg_error_2(self, fit_mock):
         # Check that training does not stop on a linalg error during hyperparameter optimization
 
         class Dummy:
             counter = 0
 
-            def __call__(self, X, eval_gradient=True):
+            def __call__(self, X, eval_gradient=True, clone_kernel=True):
                 # If this is not aligned with the GP an error will be raised that None is not iterable
                 if self.counter == 13:
                     return None
@@ -294,6 +291,19 @@ class TestGP(unittest.TestCase):
         # There's a slight difference between my local installation and travis
         self.assertLess(abs(var_hat[0][0] - 1121.8409184001594), 2)
 
+        # test other covariance results
+        _, var_fc = model.predict(X, cov_return_type='full_cov')
+        self.assertEqual(var_fc.shape, (8, 8))
+        _, var_sd = model.predict(X, cov_return_type='diagonal_std')
+        self.assertEqual(var_sd.shape, (8, 1))
+        _, var_no = model.predict(np.array([[10, 10, 10]]), cov_return_type=None)
+        self.assertIsNone(var_no)
+        # check values
+        _, var_fc = model.predict(np.array([[10, 10, 10]]), cov_return_type='full_cov')
+        self.assertAlmostEqual(var_fc[0][0], var_hat[0][0])
+        _, var_sd = model.predict(np.array([[10, 10, 10]]), cov_return_type='diagonal_std')
+        self.assertAlmostEqual(var_sd[0][0] ** 2, var_hat[0][0])
+
     def test_gp_on_sklearn_data(self):
         X, y = sklearn.datasets.load_boston(return_X_y=True)
         # Normalize such that the bounds in get_gp (10) hold
