@@ -1,6 +1,5 @@
 import unittest
 
-from nose.plugins.attrib import attr
 from scipy.optimize import fmin_l_bfgs_b
 import shutil
 from smac.facade.func_facade import fmin_smac
@@ -21,7 +20,6 @@ class TestSMACFacade(unittest.TestCase):
             if output_dir:
                 shutil.rmtree(output_dir, ignore_errors=True)
 
-    @attr('slow')
     def test_func_smac(self):
         func = rosenbrock_2d
         x0 = [-3, -4]
@@ -39,8 +37,8 @@ class TestSMACFacade(unittest.TestCase):
     def test_parameter_order(self):
         def func(x):
             for i in range(len(x)):
-                self.assertLess(i - 1, x[i])
-                self.assertGreater(i, x[i])
+                self.assertLessEqual(i - 1, x[i])
+                self.assertGreaterEqual(i, x[i])
             return 1
 
         default = [i - 0.5 for i in range(10)]
@@ -51,4 +49,3 @@ class TestSMACFacade(unittest.TestCase):
                                maxfun=1)
 
         self.output_dirs.append(smac.scenario.output_dir)
-
