@@ -11,8 +11,8 @@ class TestSobol(unittest.TestCase):
 
     def test_sobol(self):
         cs = ConfigurationSpace()
-        for i in range(40):
-            cs.add_hyperparameter(UniformFloatHyperparameter('x%d' % (i + 1), 0, 1))
+        hyperparameters = [UniformFloatHyperparameter('x%d' % (i + 1), 0, 1) for i in range(21201)]
+        cs.add_hyperparameters(hyperparameters)
 
         sobol_kwargs = dict(
             rng=np.random.RandomState(1),
@@ -28,10 +28,10 @@ class TestSobol(unittest.TestCase):
             **sobol_kwargs
         ).select_configurations()
 
-        cs.add_hyperparameter(UniformFloatHyperparameter('x41', 0, 1))
+        cs.add_hyperparameter(UniformFloatHyperparameter('x21202', 0, 1))
         with self.assertRaisesRegex(
                 Exception,
-                "('NoneType' object is not iterable)|(cannot unpack non-iterable NoneType object)",
+                "Maximum supported dimensionality is 21201.",
         ):
             SobolDesign(
                 cs=cs,
