@@ -5,6 +5,8 @@ import unittest
 
 from ConfigSpace import Configuration, ConfigurationSpace
 from ConfigSpace.hyperparameters import UniformIntegerHyperparameter
+import numpy as np
+import pynisher
 
 from smac.tae import StatusType
 from smac.runhistory.runhistory import RunHistory
@@ -251,6 +253,29 @@ class RunhistoryTest(unittest.TestCase):
 
             os.remove(path)
 
+    def test_add_json_serializable(self):
+        """Test if entries added to the runhistory are correctly checked for serializability."""
+        rh = RunHistory()
+        cs = get_config_space()
+        config = cs.sample_configuration()
+
+        rh.add(config, 0.0, 0.0, StatusType.SUCCESS, None, None, 0.0, 0.0, 0.0, None)
+        rh.add(config, 0.0, 0.0, StatusType.SUCCESS, None, None, 0.0, 0.0, 0.0, {})
+
+        with self.assertRaisesRegex(
+            ValueError,
+            r"Cannot add cost: 0\.0 of type <class 'numpy\.float32'> to runhistory because "
+            r"it raises an error during JSON encoding"
+        ):
+            rh.add(config, np.float32(0.0), 0.0, StatusType.SUCCESS, None, None, 0.0, 0.0, 0.0, None)
+        with self.assertRaisesRegex(
+            ValueError,
+            r"Cannot add additional_info: \{'error': <class 'pynisher\.limit_function_call\.AnythingException'>\} "
+            r"of type <class 'dict'> to runhistory because it raises an error during JSON encoding",
+        ):
+            rh.add(config, 0.0, 0.0, StatusType.SUCCESS, None, None, 0.0, 0.0, 0.0,
+                   {'error': pynisher.AnythingException})
+
 
 if __name__ == "__main__":
     unittest.main()
