@@ -9,7 +9,7 @@ import shlex
 
 from smac.configspace import ConfigurationSpace
 from smac.tae.execute_ta_run_aclib import ExecuteTARunAClib
-from smac.tae.execute_ta_run import StatusType
+from smac.tae import StatusType
 from smac.scenario.scenario import Scenario
 from smac.stats.stats import Stats
 
